/* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*- */
/*
 * fische-3.1
 * Copyright (C) Marcel Ebmer 2009-2010 <marcel@26elf.at>
 *
 * fische-3.1 is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 2 of the License, or
 * (at your option) any later version.
 *
 * fische-3.1 is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <cstdlib>
#include <iostream>
#include "vectorfield.h"

using namespace std;

VectorField* VectorField::ms_instance = 0;
const double Pi = acos (-1);

VectorField::VectorField (int x, int y) {
	__xRes = x;
	__yRes = y;
	__xCenter = __xRes / 2;
	__yCenter = __yRes / 2;

	__data = NULL;

	int n = 0;
	uint16_t* f = fillField (n);
	if (f) cout << "\n* calculating vectorfield: 1" << endl;
	while (f) {
		__field.push_back (f);
		n++;
		f = fillField (n);
		if (f) 	cout << "* calculating vectorfield: " << n + 1 << endl;
	}

	if (__field.empty() ) {
		cerr << "ERROR: unable to calculate any vectorfields. out of memory?" << endl;
		exit (EXIT_FAILURE);
	}

	__data = __field.at (13);
	cout << "* successfully calculated " << __field.size() << " different vectorfields\n" << endl;
}

VectorField::~VectorField() {
	for (unsigned int i = 0; i < __field.size(); i--) {
		free (__field[i]);
	}
	__field.clear();
}

VectorField* VectorField::instance (int x, int y) {
	if (!ms_instance) ms_instance = new VectorField (x, y);
	return ms_instance;
}

void VectorField::release() {
	delete ms_instance;
	ms_instance = 0;
}

char* VectorField::get() {
	return (char*) __data;
}

void VectorField::change() {
	int n = rand() % __field.size();
	while (__data == __field[n]) n = rand() % __field.size();
	__data = __field[n];
}

int VectorField::width() const {
	return __xRes;
}

int VectorField::height() const {
	return __yRes;
}

void VectorField::randomize (Vector& vec, int n) {
	return;
	if (n % 2 == 0) n++;
	Vector randv ( (rand() % n) - n / 2, (rand() % n) - n / 2);
	vec += randv;
}

uint16_t* VectorField::fillField (int f) {
	uint16_t* field = (uint16_t*) malloc (__xRes * __yRes * 2);
	if (!field) return NULL;
	for (int x = 0; x < __xRes; x++) {
		for (int y = 0; y < __yRes; y++) {
			uint16_t* vector = field + x + y * __xRes;
			Vector p (x - __xCenter, y - __yCenter);
			Vector e = p.singleUnit();
			Vector n = e.normal();
			double r = p.length();
			Vector v;
			switch (f) {
				case 0:
					// linear vectors showing away from a horizontal mirror axis
					v = Vector (0, (y < __yCenter) ? (yMax() / 100) : (-yMax() / 100) );
					break;
				case 1:
					// linear vectors showing away from a vertical mirror axis
					v = Vector ( (x < __xCenter) ? (xMax() / 100) : (-xMax() / 100), 0);
					break;
				case 2:
					// radial vectors showing away from the center
					v = e * -r / 25;
					break;
				case 3:
					// tangential vectors (right)
					v = n * r / 25;
					break;
				case 4:
					// tangential vectors (left)
					v = n * -r / 25;
					break;
				case 5:
					// tangential-radial vectors (left)
					v = e * -r / 25 - n * r / 25;
					break;
				case 6:
					// tangential-radial vectors (right)
					v = e * -r / 25 + n * r / 25;
					break;
				case 7: {
					// tree rings
					double dv = cos (Pi * 20 * r / dimension() );
					v = e * 5 * dv;
					break;
				}
				case 8: {
					// hyperbolic vectors
					Vector _e (e.y(), e.x() );
					v = _e * -r / 25;
					break;
				}
				case 9: {
					// hyperbolic vectors
					Vector _e (e.y(), e.x() );
					v = _e * r / 25;
					break;
				}
				case 10:
					// purely random
					v = Vector (rand() % 21 - 10, rand() % 21 - 10);
					break;
				case 11: {
					// sphere
					double dv = cos (Pi * r / dimension() );
					v = e * -r * dv / 15;
					break;
				}
				case 12: {
					// sine distortion
					double dv = sin (Pi * 6 * r / dimension() );
					v = e * -r * dv / 25;
					break;
				}
				case 13:
					// black hole
					v = n * dimension() / 50 + e * r / 25;
					if (r < 10) v = Vector (0, 0);
					break;
				case 14:
					// black hole
					v = n * -dimension() / 50 + e * r / 25;
					if (r < 10) v = Vector (0, 0);
					break;

				default:
					// index too high. return nothing.
					free (field);
					return NULL;
			}
			randomize (v, 2);
			validate (v, x, y);
			*vector = v.toUint16();
		}
	}
	return field;
}

int VectorField::dimension() const {
	return (__xRes > __yRes) ? __yRes : __xRes;
}
